@@ -25,41 +25,38 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+from u2flib_host.yubicommon.compat import text_type
+
 from base64 import urlsafe_b64decode, urlsafe_b64encode
 from hashlib import sha256
 
 __all__ = [
     'u2str',
     'websafe_encode',
-    'websafe_decode',
-    'H'
+    'websafe_decode'
 ]
 
 
 def u2str(data):
-    """Recursivly converts unicode object to UTF-8 formatted strings."""
+    """Recursively converts unicode objects to UTF-8 encoded byte strings."""
     if isinstance(data, dict):
-        return {u2str(k): u2str(v) for k, v in data.iteritems()}
+        return {u2str(k): u2str(v) for k, v in data.items()}
     elif isinstance(data, list):
         return [u2str(x) for x in data]
-    elif isinstance(data, unicode):
+    elif isinstance(data, text_type):
         return data.encode('utf-8')
     else:
         return data
 
 
 def websafe_decode(data):
-    if isinstance(data, unicode):
-        data = data.encode('utf-8')
-    data += '=' * (-len(data) % 4)
+    if isinstance(data, text_type):
+        data = data.encode('ascii')
+    data += b'=' * (-len(data) % 4)
     return urlsafe_b64decode(data)
 
 
 def websafe_encode(data):
-    return urlsafe_b64encode(data).replace('=', '')
-
-
-def H(data):
-    f = sha256()
-    f.update(data)
-    return f.digest()
+    if isinstance(data, text_type):
+        data = data.encode('ascii')
+    return urlsafe_b64encode(data).replace(b'=', b'').decode('ascii')
